part of model_notifier;

abstract class ValueModel<T> extends Model<T> {
  ValueModel(T value)
      : _value = value,
        super();

  StreamController<T>? _streamController;

  /// Generates a stream.
  ///
  /// Whenever a value is updated, the value of the stream is also updated.
  ///
  /// The stream is also closed when the ValueModel is [dispose()].
  Stream<T> toStream() {
    if (_streamController == null) {
      _streamController = StreamController<T>();
      _streamController!.sink.add(value);
    }
    return _streamController!.stream;
  }

  /// Discards any resources used by the object. After this is called, the
  /// object is not in a usable state and should be discarded (calls to
  /// [addListener] and [removeListener] will throw after the object is
  /// disposed).
  ///
  /// This method should only be called by the object's owner.
  @override
  @protected
  @mustCallSuper
  void dispose() {
    super.dispose();
    _streamController?.close();
  }

  /// Call all the registered listeners.
  ///
  /// Call this method whenever the object changes, to notify any clients the
  /// object may have changed. Listeners that are added during this iteration
  /// will not be visited. Listeners that are removed during this iteration will
  /// not be visited after they are removed.
  ///
  /// Exceptions thrown by listeners will be caught and reported using
  /// [FlutterError.reportError].
  ///
  /// This method must not be called after [dispose] has been called.
  ///
  /// Surprising behavior can result when reentrantly removing a listener (e.g.
  /// in response to a notification) that has been registered multiple times.
  /// See the discussion at [removeListener].
  @override
  @protected
  void notifyListeners() {
    super.notifyListeners();
    _streamController?.sink.add(value);
  }

  /// The current value stored in this notifier.
  ///
  /// When the value is replaced with something that is not equal to the old
  /// value as evaluated by the equality operator ==, this class notifies its
  /// listeners.
  T get value => _value;
  T _value;
  set value(T newValue) {
    if (_value == newValue) {
      return;
    }
    _value = newValue;
    if (notifyOnChangeValue) {
      notifyListeners();
    }
  }

  bool get notifyOnChangeValue => true;

  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  bool operator ==(Object other) => hashCode == other.hashCode;

  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  int get hashCode => value?.hashCode ?? super.hashCode;

  @override
  String toString() => "${describeIdentity(this)}($value)";
}
